@@ -1,9 +1,6 @@
-from __future__ import absolute_import, unicode_literals
-
 import threading
 import unittest
-
-import mock
+from unittest import mock
 
 import pykka
 
@@ -21,35 +18,35 @@ from tests import dummy_audio, path_to_data_dir
 
 class BaseTest(unittest.TestCase):
     config = {
-        'audio': {
-            'buffer_time': None,
-            'mixer': 'fakemixer track_max_volume=65536',
-            'mixer_track': None,
-            'mixer_volume': None,
-            'output': 'testoutput',
-            'visualizer': None,
+        "audio": {
+            "buffer_time": None,
+            "mixer": "fakemixer track_max_volume=65536",
+            "mixer_track": None,
+            "mixer_volume": None,
+            "output": "testoutput",
+            "visualizer": None,
         }
     }
 
-    uris = [path.path_to_uri(path_to_data_dir('song1.wav')),
-            path.path_to_uri(path_to_data_dir('song2.wav'))]
+    uris = [
+        path.path_to_uri(path_to_data_dir("song1.wav")),
+        path.path_to_uri(path_to_data_dir("song2.wav")),
+    ]
 
     audio_class = audio.Audio
 
     def setUp(self):  # noqa: N802
         config = {
-            'audio': {
-                'buffer_time': None,
-                'mixer': 'foomixer',
-                'mixer_volume': None,
-                'output': 'testoutput',
-                'visualizer': None,
-            },
-            'proxy': {
-                'hostname': '',
+            "audio": {
+                "buffer_time": None,
+                "mixer": "foomixer",
+                "mixer_volume": None,
+                "output": "testoutput",
+                "visualizer": None,
             },
+            "proxy": {"hostname": ""},
         }
-        self.song_uri = path.path_to_uri(path_to_data_dir('song1.wav'))
+        self.song_uri = path.path_to_uri(path_to_data_dir("song1.wav"))
         self.audio = self.audio_class.start(config=config, mixer=None).proxy()
 
     def tearDown(self):  # noqa
@@ -62,7 +59,7 @@ class BaseTest(unittest.TestCase):
         pass
 
 
-class DummyMixin(object):
+class DummyMixin:
     audio_class = dummy_audio.DummyAudio
 
     def possibly_trigger_fake_playback_error(self, uri):
@@ -75,30 +72,29 @@ class DummyMixin(object):
 
 
 class AudioTest(BaseTest):
-
     def test_start_playback_existing_file(self):
         self.audio.prepare_change()
         self.audio.set_uri(self.uris[0])
-        self.assertTrue(self.audio.start_playback().get())
+        assert self.audio.start_playback().get()
 
     def test_start_playback_non_existing_file(self):
-        self.possibly_trigger_fake_playback_error(self.uris[0] + 'bogus')
+        self.possibly_trigger_fake_playback_error(self.uris[0] + "bogus")
 
         self.audio.prepare_change()
-        self.audio.set_uri(self.uris[0] + 'bogus')
-        self.assertFalse(self.audio.start_playback().get())
+        self.audio.set_uri(self.uris[0] + "bogus")
+        assert not self.audio.start_playback().get()
 
     def test_pause_playback_while_playing(self):
         self.audio.prepare_change()
         self.audio.set_uri(self.uris[0])
         self.audio.start_playback()
-        self.assertTrue(self.audio.pause_playback().get())
+        assert self.audio.pause_playback().get()
 
     def test_stop_playback_while_playing(self):
         self.audio.prepare_change()
         self.audio.set_uri(self.uris[0])
         self.audio.start_playback()
-        self.assertTrue(self.audio.stop_playback().get())
+        assert self.audio.stop_playback().get()
 
     @unittest.SkipTest
     def test_deliver_data(self):
@@ -131,7 +127,7 @@ class AudioDummyTest(DummyMixin, AudioTest):
 
 class DummyAudioListener(pykka.ThreadingActor, audio.AudioListener):
     def __init__(self):
-        super(DummyAudioListener, self).__init__()
+        super().__init__()
         self.events = []
         self.waiters = {}
 
@@ -152,20 +148,19 @@ class DummyAudioListener(pykka.ThreadingActor, audio.AudioListener):
 
 
 class AudioEventTest(BaseTest):
-
     def setUp(self):  # noqa: N802
-        super(AudioEventTest, self).setUp()
+        super().setUp()
         self.audio.enable_sync_handler().get()
         self.listener = DummyAudioListener.start().proxy()
 
     def tearDown(self):  # noqa: N802
-        super(AudioEventTest, self).tearDown()
+        super().tearDown()
 
     def assertEvent(self, event, **kwargs):  # noqa: N802
-        self.assertIn((event, kwargs), self.listener.get_events().get())
+        assert (event, kwargs) in self.listener.get_events().get()
 
     def assertNotEvent(self, event, **kwargs):  # noqa: N802
-        self.assertNotIn((event, kwargs), self.listener.get_events().get())
+        assert (event, kwargs) not in self.listener.get_events().get()
 
     # TODO: test without uri set, with bad uri and gapless...
     # TODO: playing->playing triggered by seek should be removed
@@ -178,8 +173,12 @@ class AudioEventTest(BaseTest):
         self.audio.start_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('state_changed', old_state=PlaybackState.STOPPED,
-                         new_state=PlaybackState.PLAYING, target_state=None)
+        self.assertEvent(
+            "state_changed",
+            old_state=PlaybackState.STOPPED,
+            new_state=PlaybackState.PLAYING,
+            target_state=None,
+        )
 
     def test_state_change_stopped_to_paused_event(self):
         self.audio.prepare_change()
@@ -187,8 +186,12 @@ class AudioEventTest(BaseTest):
         self.audio.pause_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('state_changed', old_state=PlaybackState.STOPPED,
-                         new_state=PlaybackState.PAUSED, target_state=None)
+        self.assertEvent(
+            "state_changed",
+            old_state=PlaybackState.STOPPED,
+            new_state=PlaybackState.PAUSED,
+            target_state=None,
+        )
 
     def test_state_change_paused_to_playing_event(self):
         self.audio.prepare_change()
@@ -200,8 +203,12 @@ class AudioEventTest(BaseTest):
         self.audio.start_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('state_changed', old_state=PlaybackState.PAUSED,
-                         new_state=PlaybackState.PLAYING, target_state=None)
+        self.assertEvent(
+            "state_changed",
+            old_state=PlaybackState.PAUSED,
+            new_state=PlaybackState.PLAYING,
+            target_state=None,
+        )
 
     def test_state_change_paused_to_stopped_event(self):
         self.audio.prepare_change()
@@ -213,8 +220,12 @@ class AudioEventTest(BaseTest):
         self.audio.stop_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('state_changed', old_state=PlaybackState.PAUSED,
-                         new_state=PlaybackState.STOPPED, target_state=None)
+        self.assertEvent(
+            "state_changed",
+            old_state=PlaybackState.PAUSED,
+            new_state=PlaybackState.STOPPED,
+            target_state=None,
+        )
 
     def test_state_change_playing_to_paused_event(self):
         self.audio.prepare_change()
@@ -226,8 +237,12 @@ class AudioEventTest(BaseTest):
         self.audio.pause_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('state_changed', old_state=PlaybackState.PLAYING,
-                         new_state=PlaybackState.PAUSED, target_state=None)
+        self.assertEvent(
+            "state_changed",
+            old_state=PlaybackState.PLAYING,
+            new_state=PlaybackState.PAUSED,
+            target_state=None,
+        )
 
     def test_state_change_playing_to_stopped_event(self):
         self.audio.prepare_change()
@@ -239,8 +254,12 @@ class AudioEventTest(BaseTest):
         self.audio.stop_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('state_changed', old_state=PlaybackState.PLAYING,
-                         new_state=PlaybackState.STOPPED, target_state=None)
+        self.assertEvent(
+            "state_changed",
+            old_state=PlaybackState.PLAYING,
+            new_state=PlaybackState.STOPPED,
+            target_state=None,
+        )
 
     def test_stream_changed_event_on_playing(self):
         self.audio.prepare_change()
@@ -251,7 +270,7 @@ class AudioEventTest(BaseTest):
         # Since we are going from stopped to playing, the state change is
         # enough to ensure the stream changed.
         self.audio.wait_for_state_change().get()
-        self.assertEvent('stream_changed', uri=self.uris[0])
+        self.assertEvent("stream_changed", uri=self.uris[0])
 
     def test_stream_changed_event_on_multiple_changes(self):
         self.audio.prepare_change()
@@ -260,14 +279,14 @@ class AudioEventTest(BaseTest):
         self.audio.start_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('stream_changed', uri=self.uris[0])
+        self.assertEvent("stream_changed", uri=self.uris[0])
 
         self.audio.prepare_change()
         self.audio.set_uri(self.uris[1])
         self.audio.pause_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('stream_changed', uri=self.uris[1])
+        self.assertEvent("stream_changed", uri=self.uris[1])
 
     def test_stream_changed_event_on_playing_to_paused(self):
         self.audio.prepare_change()
@@ -276,13 +295,13 @@ class AudioEventTest(BaseTest):
         self.audio.start_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('stream_changed', uri=self.uris[0])
+        self.assertEvent("stream_changed", uri=self.uris[0])
 
         self.listener.clear_events()
         self.audio.pause_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertNotEvent('stream_changed', uri=self.uris[0])
+        self.assertNotEvent("stream_changed", uri=self.uris[0])
 
     def test_stream_changed_event_on_paused_to_stopped(self):
         self.audio.prepare_change()
@@ -294,7 +313,7 @@ class AudioEventTest(BaseTest):
         self.audio.stop_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('stream_changed', uri=None)
+        self.assertEvent("stream_changed", uri=None)
 
     def test_position_changed_on_pause(self):
         self.audio.prepare_change()
@@ -303,7 +322,7 @@ class AudioEventTest(BaseTest):
         self.audio.wait_for_state_change()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('position_changed', position=0)
+        self.assertEvent("position_changed", position=0)
 
     def test_stream_changed_event_on_paused_to_playing(self):
         self.audio.prepare_change()
@@ -312,13 +331,13 @@ class AudioEventTest(BaseTest):
         self.audio.pause_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('stream_changed', uri=self.uris[0])
+        self.assertEvent("stream_changed", uri=self.uris[0])
 
         self.listener.clear_events()
         self.audio.start_playback()
 
         self.audio.wait_for_state_change().get()
-        self.assertNotEvent('stream_changed', uri=self.uris[0])
+        self.assertNotEvent("stream_changed", uri=self.uris[0])
 
     def test_position_changed_on_play(self):
         self.audio.prepare_change()
@@ -327,7 +346,7 @@ class AudioEventTest(BaseTest):
         self.audio.wait_for_state_change()
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('position_changed', position=0)
+        self.assertEvent("position_changed", position=0)
 
     def test_position_changed_on_seek_while_stopped(self):
         self.audio.prepare_change()
@@ -335,7 +354,7 @@ class AudioEventTest(BaseTest):
         self.audio.set_position(2000)
 
         self.audio.wait_for_state_change().get()
-        self.assertNotEvent('position_changed', position=0)
+        self.assertNotEvent("position_changed", position=0)
 
     def test_position_changed_on_seek_after_play(self):
         self.audio.prepare_change()
@@ -347,7 +366,7 @@ class AudioEventTest(BaseTest):
         self.audio.set_position(2000)
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('position_changed', position=2000)
+        self.assertEvent("position_changed", position=2000)
 
     def test_position_changed_on_seek_after_pause(self):
         self.audio.prepare_change()
@@ -359,7 +378,7 @@ class AudioEventTest(BaseTest):
         self.audio.set_position(2000)
 
         self.audio.wait_for_state_change().get()
-        self.assertEvent('position_changed', position=2000)
+        self.assertEvent("position_changed", position=2000)
 
     def test_tags_changed_on_playback(self):
         self.audio.prepare_change()
@@ -367,7 +386,7 @@ class AudioEventTest(BaseTest):
         self.audio.start_playback()
         self.audio.wait_for_state_change().get()
 
-        self.assertEvent('tags_changed', tags=mock.ANY)
+        self.assertEvent("tags_changed", tags=mock.ANY)
 
     # Unlike the other events, having the state changed done is not
     # enough to ensure our event is called. So we setup a threading
@@ -375,7 +394,7 @@ class AudioEventTest(BaseTest):
     # completes.
 
     def test_stream_changed_event_on_paused(self):
-        event = self.listener.wait('stream_changed').get()
+        event = self.listener.wait("stream_changed").get()
 
         self.audio.prepare_change()
         self.audio.set_uri(self.uris[0])
@@ -383,12 +402,12 @@ class AudioEventTest(BaseTest):
         self.audio.wait_for_state_change().get()
 
         if not event.wait(timeout=1.0):
-            self.fail('Stream changed not reached within deadline')
+            self.fail("Stream changed not reached within deadline")
 
-        self.assertEvent('stream_changed', uri=self.uris[0])
+        self.assertEvent("stream_changed", uri=self.uris[0])
 
     def test_reached_end_of_stream_event(self):
-        event = self.listener.wait('reached_end_of_stream').get()
+        event = self.listener.wait("reached_end_of_stream").get()
 
         self.audio.prepare_change()
         self.audio.set_uri(self.uris[0])
@@ -397,13 +416,13 @@ class AudioEventTest(BaseTest):
 
         self.possibly_trigger_fake_about_to_finish()
         if not event.wait(timeout=1.0):
-            self.fail('End of stream not reached within deadline')
+            self.fail("End of stream not reached within deadline")
 
-        self.assertFalse(self.audio.get_current_tags().get())
+        assert not self.audio.get_current_tags().get()
 
     def test_gapless(self):
         uris = self.uris[1:]
-        event = self.listener.wait('reached_end_of_stream').get()
+        event = self.listener.wait("reached_end_of_stream").get()
 
         def callback():
             if uris:
@@ -421,27 +440,27 @@ class AudioEventTest(BaseTest):
         self.possibly_trigger_fake_about_to_finish()
         self.audio.wait_for_state_change().get()
         if not event.wait(timeout=1.0):
-            self.fail('EOS not received')
+            self.fail("EOS not received")
 
         # Check that both uris got played
-        self.assertEvent('stream_changed', uri=self.uris[0])
-        self.assertEvent('stream_changed', uri=self.uris[1])
+        self.assertEvent("stream_changed", uri=self.uris[0])
+        self.assertEvent("stream_changed", uri=self.uris[1])
 
         # Check that events counts check out.
         keys = [k for k, v in self.listener.get_events().get()]
-        self.assertEqual(2, keys.count('stream_changed'))
-        self.assertEqual(2, keys.count('position_changed'))
-        self.assertEqual(1, keys.count('state_changed'))
-        self.assertEqual(1, keys.count('reached_end_of_stream'))
+        assert keys.count("stream_changed") == 2
+        assert keys.count("position_changed") == 2
+        assert keys.count("state_changed") == 1
+        assert keys.count("reached_end_of_stream") == 1
 
         # TODO: test tag states within gaples
 
     # TODO: this does not belong in this testcase
     def test_current_tags_are_blank_to_begin_with(self):
-        self.assertFalse(self.audio.get_current_tags().get())
+        assert not self.audio.get_current_tags().get()
 
     def test_current_tags_blank_after_end_of_stream(self):
-        event = self.listener.wait('reached_end_of_stream').get()
+        event = self.listener.wait("reached_end_of_stream").get()
 
         self.audio.prepare_change()
         self.audio.set_uri(self.uris[0])
@@ -451,12 +470,12 @@ class AudioEventTest(BaseTest):
         self.audio.wait_for_state_change().get()
 
         if not event.wait(timeout=1.0):
-            self.fail('EOS not received')
+            self.fail("EOS not received")
 
-        self.assertFalse(self.audio.get_current_tags().get())
+        assert not self.audio.get_current_tags().get()
 
     def test_current_tags_stored(self):
-        event = self.listener.wait('reached_end_of_stream').get()
+        event = self.listener.wait("reached_end_of_stream").get()
         tags = []
 
         def callback():
@@ -472,9 +491,9 @@ class AudioEventTest(BaseTest):
         self.audio.wait_for_state_change().get()
 
         if not event.wait(timeout=1.0):
-            self.fail('EOS not received')
+            self.fail("EOS not received")
 
-        self.assertTrue(tags[0])
+        assert tags[0]
 
     # TODO: test that we reset when we expect between songs
 
@@ -486,12 +505,11 @@ class AudioDummyEventTest(DummyMixin, AudioEventTest):
 
 # TODO: move to mixer tests...
 class MixerTest(BaseTest):
-
     @unittest.SkipTest
     def test_set_mute(self):
         for value in (True, False):
-            self.assertTrue(self.audio.set_mute(value).get())
-            self.assertEqual(value, self.audio.get_mute().get())
+            assert self.audio.set_mute(value).get()
+            assert self.audio.get_mute().get() == value
 
     @unittest.SkipTest
     def test_set_state_encapsulation(self):
@@ -507,56 +525,61 @@ class MixerTest(BaseTest):
 
 
 class AudioStateTest(unittest.TestCase):
-
     def setUp(self):  # noqa: N802
         self.audio = audio.Audio(config=None, mixer=None)
 
     def test_state_starts_as_stopped(self):
-        self.assertEqual(audio.PlaybackState.STOPPED, self.audio.state)
+        assert audio.PlaybackState.STOPPED == self.audio.state
 
     def test_state_does_not_change_when_in_gst_ready_state(self):
         self.audio._handler.on_playbin_state_changed(
-            Gst.State.NULL, Gst.State.READY, Gst.State.VOID_PENDING)
+            Gst.State.NULL, Gst.State.READY, Gst.State.VOID_PENDING
+        )
 
-        self.assertEqual(audio.PlaybackState.STOPPED, self.audio.state)
+        assert audio.PlaybackState.STOPPED == self.audio.state
 
     def test_state_changes_from_stopped_to_playing_on_play(self):
         self.audio._handler.on_playbin_state_changed(
-            Gst.State.NULL, Gst.State.READY, Gst.State.PLAYING)
+            Gst.State.NULL, Gst.State.READY, Gst.State.PLAYING
+        )
         self.audio._handler.on_playbin_state_changed(
-            Gst.State.READY, Gst.State.PAUSED, Gst.State.PLAYING)
+            Gst.State.READY, Gst.State.PAUSED, Gst.State.PLAYING
+        )
         self.audio._handler.on_playbin_state_changed(
-            Gst.State.PAUSED, Gst.State.PLAYING, Gst.State.VOID_PENDING)
+            Gst.State.PAUSED, Gst.State.PLAYING, Gst.State.VOID_PENDING
+        )
 
-        self.assertEqual(audio.PlaybackState.PLAYING, self.audio.state)
+        assert audio.PlaybackState.PLAYING == self.audio.state
 
     def test_state_changes_from_playing_to_paused_on_pause(self):
         self.audio.state = audio.PlaybackState.PLAYING
 
         self.audio._handler.on_playbin_state_changed(
-            Gst.State.PLAYING, Gst.State.PAUSED, Gst.State.VOID_PENDING)
+            Gst.State.PLAYING, Gst.State.PAUSED, Gst.State.VOID_PENDING
+        )
 
-        self.assertEqual(audio.PlaybackState.PAUSED, self.audio.state)
+        assert audio.PlaybackState.PAUSED == self.audio.state
 
     def test_state_changes_from_playing_to_stopped_on_stop(self):
         self.audio.state = audio.PlaybackState.PLAYING
 
         self.audio._handler.on_playbin_state_changed(
-            Gst.State.PLAYING, Gst.State.PAUSED, Gst.State.NULL)
+            Gst.State.PLAYING, Gst.State.PAUSED, Gst.State.NULL
+        )
         self.audio._handler.on_playbin_state_changed(
-            Gst.State.PAUSED, Gst.State.READY, Gst.State.NULL)
+            Gst.State.PAUSED, Gst.State.READY, Gst.State.NULL
+        )
         # We never get the following call, so the logic must work without it
         # self.audio._handler.on_playbin_state_changed(
         #     Gst.State.READY, Gst.State.NULL, Gst.State.VOID_PENDING)
 
-        self.assertEqual(audio.PlaybackState.STOPPED, self.audio.state)
+        assert audio.PlaybackState.STOPPED == self.audio.state
 
 
 class AudioBufferingTest(unittest.TestCase):
-
     def setUp(self):  # noqa: N802
         self.audio = audio.Audio(config=None, mixer=None)
-        self.audio._playbin = mock.Mock(spec=['set_state'])
+        self.audio._playbin = mock.Mock(spec=["set_state"])
 
     def test_pause_when_buffer_empty(self):
         playbin = self.audio._playbin
@@ -566,7 +589,7 @@ class AudioBufferingTest(unittest.TestCase):
 
         self.audio._handler.on_buffering(0)
         playbin.set_state.assert_called_with(Gst.State.PAUSED)
-        self.assertTrue(self.audio._buffering)
+        assert self.audio._buffering
 
     def test_stay_paused_when_buffering_finished(self):
         playbin = self.audio._playbin
@@ -575,8 +598,8 @@ class AudioBufferingTest(unittest.TestCase):
         playbin.set_state.reset_mock()
 
         self.audio._handler.on_buffering(100)
-        self.assertEqual(playbin.set_state.call_count, 0)
-        self.assertFalse(self.audio._buffering)
+        assert playbin.set_state.call_count == 0
+        assert not self.audio._buffering
 
     def test_change_to_paused_while_buffering(self):
         playbin = self.audio._playbin
@@ -590,8 +613,8 @@ class AudioBufferingTest(unittest.TestCase):
         playbin.set_state.reset_mock()
 
         self.audio._handler.on_buffering(100)
-        self.assertEqual(playbin.set_state.call_count, 0)
-        self.assertFalse(self.audio._buffering)
+        assert playbin.set_state.call_count == 0
+        assert not self.audio._buffering
 
     def test_change_to_stopped_while_buffering(self):
         playbin = self.audio._playbin
@@ -605,4 +628,56 @@ class AudioBufferingTest(unittest.TestCase):
 
         self.audio.stop_playback()
         playbin.set_state.assert_called_with(Gst.State.NULL)
-        self.assertFalse(self.audio._buffering)
+        assert not self.audio._buffering
+
+
+class AudioLiveTest(unittest.TestCase):
+    config = {
+        "audio": {
+            "buffer_time": None,
+            "mixer": "fakemixer track_max_volume=65536",
+            "mixer_track": None,
+            "mixer_volume": None,
+            "output": "testoutput",
+            "visualizer": None,
+        }
+    }
+
+    def setUp(self):  # noqa: N802
+        config = {
+            "audio": {
+                "buffer_time": None,
+                "mixer": "foomixer",
+                "mixer_volume": None,
+                "output": "testoutput",
+                "visualizer": None,
+            },
+            "proxy": {"hostname": ""},
+        }
+        self.audio = audio.Audio(config=config, mixer=None)
+
+    def test_not_live_mode(self):
+        source = mock.MagicMock()
+
+        # Avoid appsrc.configure()
+        source.get_factory.get_name = mock.MagicMock(return_value="not_appsrc")
+
+        source.props = mock.MagicMock(spec=[])
+        self.audio._live_stream = False
+
+        self.audio._on_source_setup("dummy", source)
+
+        source.set_live.assert_not_called()
+
+    def test_live_mode(self):
+        source = mock.MagicMock()
+
+        # Avoid appsrc.configure()
+        source.get_factory.get_name = mock.MagicMock(return_value="not_appsrc")
+
+        source.props.is_live = mock.MagicMock(return_value=True)
+        self.audio._live_stream = True
+
+        self.audio._on_source_setup("dummy", source)
+
+        source.set_live.assert_called_with(True)
