@@ -1,21 +1,17 @@
-from __future__ import absolute_import, unicode_literals
-
 import unittest
-
-import mock
+from unittest import mock
 
 from mopidy import backend
 
 
 class BackendListenerTest(unittest.TestCase):
-
     def setUp(self):  # noqa: N802
         self.listener = backend.BackendListener()
 
     def test_on_event_forwards_to_specific_handler(self):
         self.listener.playlists_loaded = mock.Mock()
 
-        self.listener.on_event('playlists_loaded')
+        self.listener.on_event("playlists_loaded")
 
         self.listener.playlists_loaded.assert_called_with()
 
